@@ -49,6 +49,7 @@ pub enum RevMapping {
 
 #[allow(clippy::len_without_is_empty)]
 impl RevMapping {
+    /// Get the length of the [`RevMapping`]
     pub fn len(&self) -> usize {
         match self {
             Self::Global(_, a, _) => a.len(),
@@ -66,6 +67,20 @@ impl RevMapping {
             Self::Local(a) => a.value(idx as usize),
         }
     }
+
+    /// Categorical to str
+    ///
+    /// # Safety:
+    /// This doesn't do any bound checking
+    pub unsafe fn get_unchecked(&self, idx: u32) -> &str {
+        match self {
+            Self::Global(map, a, _) => {
+                let idx = *map.get(&idx).unwrap();
+                a.value_unchecked(idx as usize)
+            }
+            Self::Local(a) => a.value_unchecked(idx as usize),
+        }
+    }
     /// Check if the categoricals are created under the same global string cache.
     pub fn same_src(&self, other: &Self) -> bool {
         match (self, other) {
@@ -205,7 +220,7 @@ mod test {
             Some("foo"),
             Some("bar"),
         ];
-        let ca = Utf8Chunked::new_from_opt_slice("a", slice);
+        let ca = Utf8Chunked::new("a", slice);
         let out = ca.cast(&DataType::Categorical)?;
         let mut out = out.categorical().unwrap().clone();
         assert_eq!(out.categorical_map.take().unwrap().len(), 2);
@@ -224,9 +239,9 @@ mod test {
         // Check that we don't panic if we append two categorical arrays
         // build under the same string cache
         // https://github.com/pola-rs/polars/issues/1115
-        let ca1 = Utf8Chunked::new_from_opt_slice("a", slice).cast(&DataType::Categorical)?;
+        let ca1 = Utf8Chunked::new("a", slice).cast(&DataType::Categorical)?;
         let mut ca1 = ca1.categorical().unwrap().clone();
-        let ca2 = Utf8Chunked::new_from_opt_slice("a", slice).cast(&DataType::Categorical)?;
+        let ca2 = Utf8Chunked::new("a", slice).cast(&DataType::Categorical)?;
         let ca2 = ca2.categorical().unwrap();
         ca1.append(ca2);
 
