@@ -100,9 +100,13 @@ impl<'a> Iterator for CatIter<'a> {
     type Item = Option<&'a str>;
 
     fn next(&mut self) -> Option<Self::Item> {
-        self.iter
-            .next()
-            .map(|item| item.map(|idx| self.rev.get(idx)))
+        self.iter.next().map(|item| {
+            item.map(|idx| {
+                // Safety:
+                // all categories are in bound
+                unsafe { self.rev.get_unchecked(idx) }
+            })
+        })
     }
 
     fn size_hint(&self) -> (usize, Option<usize>) {
@@ -159,7 +163,7 @@ mod test {
             Some("foo"),
             Some("bar"),
         ];
-        let ca = Utf8Chunked::new_from_opt_slice("a", slice);
+        let ca = Utf8Chunked::new("a", slice);
         let ca = ca.cast(&DataType::Categorical)?;
         let ca = ca.categorical().unwrap();
 
