@@ -8,7 +8,6 @@ use std::sync::Arc;
 
 use ahash::{AHashSet, RandomState};
 use arrow::record_batch::RecordBatch;
-use itertools::Itertools;
 use rayon::prelude::*;
 
 use crate::chunked_array::ops::unique::is_unique_helper;
@@ -18,6 +17,7 @@ use crate::utils::{
     accumulate_dataframes_horizontal, accumulate_dataframes_vertical, split_ca, split_df, NoNull,
 };
 
+#[cfg(feature = "dataframe_arithmetic")]
 mod arithmetic;
 #[cfg(feature = "asof_join")]
 pub(crate) mod asof_join;
