@@ -14,6 +14,25 @@ use std::fmt;
 use std::sync::atomic::Ordering;
 use std::sync::{atomic::AtomicUsize, Arc};
 
+pub fn to_cast(df: &mut DataFrame, to_cast: &[&Field]) -> Result<()> {
+    // cast to the original dtypes in the schema
+    for fld in to_cast {
+        use DataType::*;
+        df.may_apply(fld.name(), |s| match (s.dtype(), fld.data_type()) {
+            #[cfg(feature = "temporal")]
+            (Utf8, Date) => s.utf8().unwrap().as_date(None).map(|ca| ca.into_series()),
+            #[cfg(feature = "temporal")]
+            (Utf8, Datetime) => s
+                .utf8()
+                .unwrap()
+                .as_datetime(None)
+                .map(|ca| ca.into_series()),
+            (_, dt) => s.cast(dt),
+        })?;
+    }
+    Ok(())
+}
+
 /// CSV file reader
 pub(crate) struct CoreReader<'a> {
     reader_bytes: Option<ReaderBytes<'a>>,
@@ -38,6 +57,7 @@ pub(crate) struct CoreReader<'a> {
     null_values: Option<Vec<String>>,
     predicate: Option<Arc<dyn PhysicalIoExpr>>,
     aggregate: Option<&'a [ScanAggregation]>,
+    to_cast: &'a [&'a Field],
 }
 
 impl<'a> fmt::Debug for CoreReader<'a> {
@@ -123,10 +143,16 @@ impl<'a> CoreReader<'a> {
         null_values: Option<NullValues>,
         predicate: Option<Arc<dyn PhysicalIoExpr>>,
         aggregate: Option<&'a [ScanAggregation]>,
+        to_cast: &'a [&'a Field],
     ) -> Result<CoreReader<'a>> {
         #[cfg(any(feature = "decompress", feature = "decompress-fast"))]
         let mut reader_bytes = reader_bytes;
 
+        #[cfg(not(any(feature = "decompress", feature = "decompress-fast")))]
+        if is_compressed(&reader_bytes) {
+            return Err(PolarsError::ComputeError("cannot read compressed csv file; compile with feature 'decompress' or 'decompress-fast'".into()));
+        }
+
         // check if schema should be inferred
         let delimiter = delimiter.unwrap_or(b',');
 
@@ -210,6 +236,7 @@ impl<'a> CoreReader<'a> {
             null_values,
             predicate,
             aggregate,
+            to_cast,
         })
     }
 
@@ -413,7 +440,7 @@ impl<'a> CoreReader<'a> {
                             let local_bytes = &bytes[read..stop_at_nbytes];
 
                             last_read = read;
-                            read = parse_lines(
+                            read += parse_lines(
                                 local_bytes,
                                 read,
                                 delimiter,
@@ -473,7 +500,11 @@ impl<'a> CoreReader<'a> {
                             }
                         }
 
-                        Ok(df)
+                        df.map(|mut df| {
+                            to_cast(&mut df, self.to_cast)?;
+                            Ok(df)
+                        })
+                        .transpose()
                     })
                     .collect::<Result<Vec<_>>>()
             })?;
@@ -525,7 +556,7 @@ impl<'a> CoreReader<'a> {
                             let local_bytes = &bytes[read..stop_at_nbytes];
 
                             last_read = read;
-                            read = parse_lines(
+                            read += parse_lines(
                                 local_bytes,
                                 read,
                                 delimiter,
@@ -537,15 +568,19 @@ impl<'a> CoreReader<'a> {
                                 ignore_parser_errors,
                                 // chunk size doesn't really matter anymore,
                                 // less calls if we increase the size
-                                chunk_size * 320000,
+                                usize::MAX,
                             )?;
                         }
-                        Ok(DataFrame::new_no_checks(
+
+                        let mut df = DataFrame::new_no_checks(
                             buffers
                                 .into_iter()
                                 .map(|buf| buf.into_series())
                                 .collect::<Result<_>>()?,
-                        ))
+                        );
+
+                        to_cast(&mut df, self.to_cast)?;
+                        Ok(df)
                     })
                     .collect::<Result<Vec<_>>>()
             })?;
