@@ -16,8 +16,7 @@ use crate::logical_plan::optimizer::stack_opt::{OptimizationRule, StackOptimizer
 use crate::logical_plan::optimizer::{
     predicate_pushdown::PredicatePushDown, projection_pushdown::ProjectionPushDown,
 };
-#[cfg(feature = "ipc")]
-use crate::logical_plan::IpcOptions;
+use crate::logical_plan::ScanOptions;
 use crate::physical_plan::state::ExecutionState;
 #[cfg(any(feature = "parquet", feature = "csv-file"))]
 use crate::prelude::aggregate_scan_projections::agg_projection;
@@ -296,6 +295,7 @@ impl LazyFrame {
 
     /// Create a LazyFrame directly from a parquet scan.
     #[cfg(feature = "parquet")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "parquet")))]
     pub fn scan_parquet(
         path: String,
         n_rows: Option<usize>,
@@ -311,8 +311,9 @@ impl LazyFrame {
 
     /// Create a LazyFrame directly from a ipc scan.
     #[cfg(feature = "ipc")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "ipc")))]
     pub fn scan_ipc(path: String, n_rows: Option<usize>, cache: bool) -> Result<Self> {
-        let options = IpcOptions {
+        let options = ScanOptions {
             n_rows,
             cache,
             with_columns: None,
@@ -323,6 +324,8 @@ impl LazyFrame {
     }
 
     /// Get a dot language representation of the LogicalPlan.
+    #[cfg(feature = "dot_diagram")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "dot_diagram")))]
     pub fn to_dot(&self, optimized: bool) -> Result<String> {
         let mut s = String::with_capacity(512);
 
