@@ -3,18 +3,21 @@ from datetime import date, datetime
 from typing import TYPE_CHECKING, Any, Dict, List, Optional, Sequence, Type, Union
 
 import numpy as np
-import pyarrow as pa
-import pyarrow.compute
 
-import polars as pl
+from polars import internals as pli
 from polars.datatypes import (
+    Categorical,
     DataType,
-    Date32,
-    Date64,
+    Date,
+    Datetime,
     Float32,
+    Time,
+    py_type_to_arrow_type,
+    py_type_to_dtype,
+)
+from polars.datatypes_constructor import (
     numpy_type_to_constructor,
     polars_type_to_constructor,
-    py_type_to_arrow_type,
     py_type_to_constructor,
 )
 
@@ -22,13 +25,21 @@ try:
     from polars.polars import PyDataFrame, PySeries
 
     _DOCUMENTING = False
-except ImportError:
+except ImportError:  # pragma: no cover
     _DOCUMENTING = True
-from polars.utils import coerce_arrow
 
-if TYPE_CHECKING:
+if TYPE_CHECKING:  # pragma: no cover
     import pandas as pd
+    import pyarrow as pa
 
+    _PYARROW_AVAILABLE = True
+else:
+    try:
+        import pyarrow as pa
+
+        _PYARROW_AVAILABLE = True
+    except ImportError:  # pragma: no cover
+        _PYARROW_AVAILABLE = False
 
 ################################
 # Series constructor interface #
@@ -37,7 +48,7 @@ if TYPE_CHECKING:
 
 def series_to_pyseries(
     name: str,
-    values: "pl.Series",
+    values: "pli.Series",
 ) -> "PySeries":
     """
     Construct a PySeries from a Polars Series.
@@ -46,32 +57,47 @@ def series_to_pyseries(
     return values.inner()
 
 
-def arrow_to_pyseries(name: str, values: pa.Array) -> "PySeries":
+def arrow_to_pyseries(
+    name: str, values: "pa.Array", rechunk: bool = True
+) -> "PySeries":
     """
     Construct a PySeries from an Arrow array.
     """
     array = coerce_arrow(values)
+    if hasattr(array, "num_chunks"):
+        if array.num_chunks > 1:
+            it = array.iterchunks()
+            pys = PySeries.from_arrow(name, next(it))
+            for a in it:
+                pys.append(PySeries.from_arrow(name, a))
+        else:
+            pys = PySeries.from_arrow(name, array.combine_chunks())
+
+        if rechunk:
+            pys.rechunk(in_place=True)
+
+        return pys
     return PySeries.from_arrow(name, array)
 
 
 def numpy_to_pyseries(
-    name: str, values: np.ndarray, nullable: bool = True, strict: bool = True
+    name: str, values: np.ndarray, strict: bool = True, nan_to_null: bool = False
 ) -> "PySeries":
     """
     Construct a PySeries from a numpy array.
     """
-    if not values.data.contiguous:
+    if not values.flags["C_CONTIGUOUS"]:
         values = np.array(values)
 
     if len(values.shape) == 1:
         dtype = values.dtype.type
         constructor = numpy_type_to_constructor(dtype)
         if dtype == np.float32 or dtype == np.float64:
-            return constructor(name, values, nullable)
+            return constructor(name, values, nan_to_null)
         else:
             return constructor(name, values, strict)
     else:
-        return PySeries.new_object(name, values)
+        return PySeries.new_object(name, values, strict)
 
 
 def _get_first_non_none(values: Sequence[Optional[Any]]) -> Any:
@@ -99,10 +125,10 @@ def sequence_to_pyseries(
     if dtype is not None:
         constructor = polars_type_to_constructor(dtype)
         pyseries = constructor(name, values, strict)
-        if dtype == Date32:
-            pyseries = pyseries.cast(str(pl.Date32), True)
-        elif dtype == Date64:
-            pyseries = pyseries.cast(str(pl.Date64), True)
+
+        if dtype in (Date, Datetime, Time, Categorical):
+            pyseries = pyseries.cast(str(dtype), True)
+
         return pyseries
 
     else:
@@ -110,25 +136,51 @@ def sequence_to_pyseries(
         dtype_ = type(value) if value is not None else float
 
         if dtype_ == date or dtype_ == datetime:
+            if not _PYARROW_AVAILABLE:  # pragma: no cover
+                raise ImportError(
+                    "'pyarrow' is required for converting a Sequence of date or datetime values to a PySeries."
+                )
             return arrow_to_pyseries(name, pa.array(values))
 
-        elif dtype_ == list or dtype_ == tuple or dtype_ == pl.Series:
+        elif dtype_ == list or dtype_ == tuple:
             nested_value = _get_first_non_none(value)
             nested_dtype = type(nested_value) if value is not None else float
 
-            try:
-                nested_arrow_dtype = py_type_to_arrow_type(nested_dtype)
-            except ValueError as e:
-                raise ValueError(
-                    f"Cannot construct Series from sequence of {nested_dtype}."
-                ) from e
+            # logs will show a panic if we infer wrong dtype
+            # and its hard to error from rust side
+            # to reduce the likelihood of this happening
+            # we infer the dtype of first 100 elements
+            # if all() fails, we will hit the PySeries.new_object
+            if not _PYARROW_AVAILABLE:
+                if all(
+                    isinstance(val, nested_dtype)
+                    for val in values[: min(100, len(values))]
+                ):  # pragma: no cover
+                    dtype = py_type_to_dtype(nested_dtype)
+                    try:
+                        return PySeries.new_list(name, values, dtype)
+                    except BaseException:
+                        pass
+                # pass we create an object if we get here
+            else:
+                try:
+                    nested_arrow_dtype = py_type_to_arrow_type(nested_dtype)
+                except ValueError as e:  # pragma: no cover
+                    raise ValueError(
+                        f"Cannot construct Series from sequence of {nested_dtype}."
+                    ) from e
 
-            try:
-                arrow_values = pa.array(values, pa.large_list(nested_arrow_dtype))
-                return arrow_to_pyseries(name, arrow_values)
-            # failure expected for mixed sequences like `[[12], "foo", 9]`
-            except pa.lib.ArrowInvalid:
-                return PySeries.new_object(name, values, strict)
+                try:
+                    arrow_values = pa.array(values, pa.large_list(nested_arrow_dtype))
+                    return arrow_to_pyseries(name, arrow_values)
+                except pa.lib.ArrowInvalid:
+                    pass
+
+            # Convert mixed sequences like `[[12], "foo", 9]`
+            return PySeries.new_object(name, values, strict)
+
+        elif dtype_ == pli.Series:
+            return PySeries.new_series_list(name, [v.inner() for v in values], strict)
 
         else:
             constructor = py_type_to_constructor(dtype_)
@@ -136,23 +188,44 @@ def sequence_to_pyseries(
 
 
 def _pandas_series_to_arrow(
-    values: Union["pd.Series", "pd.DatetimeIndex"], nan_to_none: bool = True
-) -> pa.Array:
+    values: Union["pd.Series", "pd.DatetimeIndex"],
+    nan_to_none: bool = True,
+    min_len: Optional[int] = None,
+) -> "pa.Array":
     """
-    Convert a pandas Series to an Arrow array.
+    Convert a pandas Series to an Arrow Array.
+
+    Parameters
+    ----------
+    values
+        Series to convert to arrow
+    nan_to_none
+        Interpret `NaN` as missing values
+    min_len
+        in case of null values, this length will be used to create a dummy f64 array (with all values set to null)
+
+    Returns
+    -------
     """
     dtype = values.dtype
     if dtype == "datetime64[ns]":
-        # We first cast to ms because that's the unit of Date64,
-        # Then we cast to via int64 to date64. Casting directly to Date64 lead to
+        # We first cast to ms because that's the unit of Datetime,
+        # Then we cast to via int64 to datetime. Casting directly to Datetime lead to
         # loss of time information https://github.com/pola-rs/polars/issues/476
         arr = pa.array(
             np.array(values.values, dtype="datetime64[ms]"), from_pandas=nan_to_none
         )
         arr = pa.compute.cast(arr, pa.int64())
-        return pa.compute.cast(arr, pa.date64())
-    elif dtype == "object" and len(values) > 0 and isinstance(values.iloc[0], str):
-        return pa.array(values, pa.large_utf8(), from_pandas=nan_to_none)
+        return pa.compute.cast(arr, pa.timestamp("ms"))
+    elif dtype == "object" and len(values) > 0:
+        if isinstance(values.iloc[0], str):
+            return pa.array(values, pa.large_utf8(), from_pandas=nan_to_none)
+
+        # array is null array, we set to a float64 array
+        if values.iloc[0] is None and min_len is not None:
+            return pa.nulls(min_len, pa.float64())
+        else:
+            return pa.array(values, from_pandas=nan_to_none)
     else:
         return pa.array(values, from_pandas=nan_to_none)
 
@@ -163,6 +236,10 @@ def pandas_to_pyseries(
     """
     Construct a PySeries from a pandas Series or DatetimeIndex.
     """
+    if not _PYARROW_AVAILABLE:  # pragma: no cover
+        raise ImportError(
+            "'pyarrow' is required when constructing a PySeries from a pandas Series."
+        )
     # TODO: Change `if not name` to `if name is not None` once name is Optional[str]
     if not name and values.name is not None:
         name = str(values.name)
@@ -179,7 +256,6 @@ def pandas_to_pyseries(
 def _handle_columns_arg(
     data: List["PySeries"],
     columns: Optional[Sequence[str]] = None,
-    nullable: bool = True,
 ) -> List["PySeries"]:
     """
     Rename data according to columns argument.
@@ -188,7 +264,7 @@ def _handle_columns_arg(
         return data
     else:
         if not data:
-            return [pl.Series(c, None, nullable=nullable).inner() for c in columns]
+            return [pli.Series(c, None).inner() for c in columns]
         elif len(data) == len(columns):
             for i, c in enumerate(columns):
                 data[i].rename(c)
@@ -200,16 +276,12 @@ def _handle_columns_arg(
 def dict_to_pydf(
     data: Dict[str, Sequence[Any]],
     columns: Optional[Sequence[str]] = None,
-    nullable: bool = True,
 ) -> "PyDataFrame":
     """
     Construct a PyDataFrame from a dictionary of sequences.
     """
-    data_series = [
-        pl.Series(name, values, nullable=nullable).inner()
-        for name, values in data.items()
-    ]
-    data_series = _handle_columns_arg(data_series, columns=columns, nullable=nullable)
+    data_series = [pli.Series(name, values).inner() for name, values in data.items()]
+    data_series = _handle_columns_arg(data_series, columns=columns)
     return PyDataFrame(data_series)
 
 
@@ -217,7 +289,6 @@ def numpy_to_pydf(
     data: np.ndarray,
     columns: Optional[Sequence[str]] = None,
     orient: Optional[str] = None,
-    nullable: bool = True,
 ) -> "PyDataFrame":
     """
     Construct a PyDataFrame from a numpy ndarray.
@@ -228,7 +299,7 @@ def numpy_to_pydf(
         data_series = []
 
     elif len(shape) == 1:
-        s = pl.Series("column_0", data, nullable=False).inner()
+        s = pli.Series("column_0", data).inner()
         data_series = [s]
 
     elif len(shape) == 2:
@@ -248,18 +319,16 @@ def numpy_to_pydf(
 
         if orient == "row":
             data_series = [
-                pl.Series(f"column_{i}", data[:, i], nullable=False).inner()
-                for i in range(shape[1])
+                pli.Series(f"column_{i}", data[:, i]).inner() for i in range(shape[1])
             ]
         else:
             data_series = [
-                pl.Series(f"column_{i}", data[i], nullable=False).inner()
-                for i in range(shape[0])
+                pli.Series(f"column_{i}", data[i]).inner() for i in range(shape[0])
             ]
     else:
         raise ValueError("A numpy array should not have more than two dimensions.")
 
-    data_series = _handle_columns_arg(data_series, columns=columns, nullable=nullable)
+    data_series = _handle_columns_arg(data_series, columns=columns)
 
     return PyDataFrame(data_series)
 
@@ -268,7 +337,6 @@ def sequence_to_pydf(
     data: Sequence[Any],
     columns: Optional[Sequence[str]] = None,
     orient: Optional[str] = None,
-    nullable: bool = True,
 ) -> "PyDataFrame":
     """
     Construct a PyDataFrame from a sequence.
@@ -277,7 +345,7 @@ def sequence_to_pydf(
     if len(data) == 0:
         data_series = []
 
-    elif isinstance(data[0], pl.Series):
+    elif isinstance(data[0], pli.Series):
         data_series = []
         for i, s in enumerate(data):
             if not s.name:  # TODO: Replace by `if s.name is None` once allowed
@@ -302,24 +370,27 @@ def sequence_to_pydf(
             return pydf
         else:
             data_series = [
-                pl.Series(f"column_{i}", data[i], nullable=nullable).inner()
-                for i in range(len(data))
+                pli.Series(f"column_{i}", data[i]).inner() for i in range(len(data))
             ]
 
     else:
-        s = pl.Series("column_0", data, nullable=nullable).inner()
+        s = pli.Series("column_0", data).inner()
         data_series = [s]
 
-    data_series = _handle_columns_arg(data_series, columns=columns, nullable=nullable)
+    data_series = _handle_columns_arg(data_series, columns=columns)
     return PyDataFrame(data_series)
 
 
 def arrow_to_pydf(
-    data: pa.Table, columns: Optional[Sequence[str]] = None, rechunk: bool = True
+    data: "pa.Table", columns: Optional[Sequence[str]] = None, rechunk: bool = True
 ) -> "PyDataFrame":
     """
     Construct a PyDataFrame from an Arrow Table.
     """
+    if not _PYARROW_AVAILABLE:  # pragma: no cover
+        raise ImportError(
+            "'pyarrow' is required when constructing a PyDataFrame from an Arrow Table."
+        )
     if columns is not None:
         try:
             data = data.rename_columns(columns)
@@ -329,25 +400,44 @@ def arrow_to_pydf(
             ) from e
 
     data_dict = {}
+    # dictionaries cannot be build in different batches (categorical does not allow that)
+    # so we rechunk them and create them separate.
+    dictionary_cols = {}
+    names = []
     for i, column in enumerate(data):
         # extract the name before casting
         if column._name is None:
             name = f"column_{i}"
         else:
             name = column._name
+        names.append(name)
 
         column = coerce_arrow(column)
-        data_dict[name] = column
+        if pa.types.is_dictionary(column.type):
+            ps = arrow_to_pyseries(name, column, rechunk)
+            dictionary_cols[i] = pli.wrap_s(ps)
+        else:
+            data_dict[name] = column
 
-    batches = pa.table(data_dict).to_batches()
-    pydf = PyDataFrame.from_arrow_record_batches(batches)
+    if len(data_dict) > 0:
+        batches = pa.table(data_dict).to_batches()
+        pydf = PyDataFrame.from_arrow_record_batches(batches)
+    else:
+        pydf = pli.DataFrame([])._df
     if rechunk:
         pydf = pydf.rechunk()
+
+    if len(dictionary_cols) > 0:
+        df = pli.wrap_df(pydf)
+        for i, s in dictionary_cols.items():
+            df[s.name] = s
+        df = df[names]
+        pydf = df._df
     return pydf
 
 
 def series_to_pydf(
-    data: "pl.Series",
+    data: "pli.Series",
     columns: Optional[Sequence[str]] = None,
 ) -> "PyDataFrame":
     """
@@ -364,9 +454,45 @@ def pandas_to_pydf(
     rechunk: bool = True,
     nan_to_none: bool = True,
 ) -> "PyDataFrame":
+    """
+    Construct a PyDataFrame from a pandas DataFrame.
+    """
+    if not _PYARROW_AVAILABLE:  # pragma: no cover
+        raise ImportError(
+            "'pyarrow' is required when constructing a PyDataFrame from a pandas DataFrame."
+        )
+    len = data.shape[0]
     arrow_dict = {
-        str(col): _pandas_series_to_arrow(data[col], nan_to_none=nan_to_none)
+        str(col): _pandas_series_to_arrow(
+            data[col], nan_to_none=nan_to_none, min_len=len
+        )
         for col in data.columns
     }
     arrow_table = pa.table(arrow_dict)
     return arrow_to_pydf(arrow_table, columns=columns, rechunk=rechunk)
+
+
+def coerce_arrow(array: "pa.Array", rechunk: bool = True) -> "pa.Array":
+    if isinstance(array, pa.TimestampArray) and array.type.tz is not None:
+        warnings.warn(
+            "Conversion of timezone aware to naive datetimes. TZ information may be lost",
+        )
+
+    # note: Decimal256 could not be cast to float
+    if isinstance(array.type, pa.Decimal128Type):
+        array = pa.compute.cast(array, pa.float64())
+
+    if hasattr(array, "num_chunks") and array.num_chunks > 1 and rechunk:
+        # small integer keys can often not be combined, so let's already cast
+        # to the uint32 used by polars
+        if pa.types.is_dictionary(array.type) and (
+            pa.types.is_int8(array.type.index_type)
+            or pa.types.is_uint8(array.type.index_type)
+            or pa.types.is_int16(array.type.index_type)
+            or pa.types.is_uint16(array.type.index_type)
+            or pa.types.is_int32(array.type.index_type)
+        ):
+            array = pa.compute.cast(
+                array, pa.dictionary(pa.uint32(), pa.large_string())
+            ).combine_chunks()
+    return array
