@@ -1027,6 +1027,46 @@ impl PyExpr {
     pub fn shuffle(&self, seed: u64) -> Self {
         self.inner.clone().shuffle(seed).into()
     }
+
+    pub fn ewm_mean(&self, alpha: f64, adjust: bool, min_periods: usize) -> Self {
+        let options = EWMOptions {
+            alpha,
+            adjust,
+            min_periods,
+        };
+        self.inner.clone().ewm_mean(options).into()
+    }
+    pub fn ewm_std(&self, alpha: f64, adjust: bool, min_periods: usize) -> Self {
+        let options = EWMOptions {
+            alpha,
+            adjust,
+            min_periods,
+        };
+        self.inner.clone().ewm_std(options).into()
+    }
+    pub fn ewm_var(&self, alpha: f64, adjust: bool, min_periods: usize) -> Self {
+        let options = EWMOptions {
+            alpha,
+            adjust,
+            min_periods,
+        };
+        self.inner.clone().ewm_var(options).into()
+    }
+    pub fn extend(&self, py: Python, value: Wrap<AnyValue>, n: usize) -> Self {
+        let value = value.into_py(py);
+        self.inner
+            .clone()
+            .apply(
+                move |s| {
+                    let gil = Python::acquire_gil();
+                    let py = gil.python();
+                    let value = value.extract::<Wrap<AnyValue>>(py).unwrap().0;
+                    s.extend(value, n)
+                },
+                GetOutput::same_type(),
+            )
+            .into()
+    }
 }
 
 impl From<dsl::Expr> for PyExpr {
