@@ -35,7 +35,7 @@ use crate::error::PyPolarsEr;
 use crate::file::get_either_file;
 use crate::prelude::{ClosedWindow, DataType, Duration, PyDataType};
 use mimalloc::MiMalloc;
-use polars::functions::diag_concat_df;
+use polars::functions::{diag_concat_df, hor_concat_df};
 use polars_core::export::arrow::io::ipc::read::read_file_metadata;
 use polars_core::prelude::IntoSeries;
 use pyo3::types::PyDict;
@@ -224,6 +224,22 @@ fn py_diag_concat_df(dfs: &PyAny) -> PyResult<PyDataFrame> {
     Ok(df.into())
 }
 
+#[pyfunction]
+fn py_hor_concat_df(dfs: &PyAny) -> PyResult<PyDataFrame> {
+    let (seq, _len) = get_pyseq(dfs)?;
+    let iter = seq.iter()?;
+
+    let dfs = iter
+        .map(|item| {
+            let item = item?;
+            get_df(item)
+        })
+        .collect::<PyResult<Vec<_>>>()?;
+
+    let df = hor_concat_df(&dfs).map_err(PyPolarsEr::from)?;
+    Ok(df.into())
+}
+
 #[pyfunction]
 fn concat_series(series: &PyAny) -> PyResult<PySeries> {
     let (seq, _len) = get_pyseq(series)?;
@@ -332,6 +348,7 @@ fn polars(_py: Python, m: &PyModule) -> PyResult<()> {
     m.add_wrapped(wrap_pyfunction!(spearman_rank_corr)).unwrap();
     m.add_wrapped(wrap_pyfunction!(map_mul)).unwrap();
     m.add_wrapped(wrap_pyfunction!(py_diag_concat_df)).unwrap();
+    m.add_wrapped(wrap_pyfunction!(py_hor_concat_df)).unwrap();
     m.add_wrapped(wrap_pyfunction!(py_datetime)).unwrap();
     m.add_wrapped(wrap_pyfunction!(py_date_range)).unwrap();
     Ok(())
