@@ -206,3 +206,94 @@ def test_date_range() -> None:
     assert result.dt[1] == datetime(1985, 1, 2, 12, 0)
     assert result.dt[2] == datetime(1985, 1, 4, 0, 0)
     assert result.dt[-1] == datetime(2015, 6, 30, 12, 0)
+
+
+def test_date_comp() -> None:
+    one = datetime(2001, 1, 1)
+    two = datetime(2001, 1, 2)
+    a = pl.Series("a", [one, two])
+
+    assert (a == one).to_list() == [True, False]
+    assert (a != one).to_list() == [False, True]
+    assert (a > one).to_list() == [False, True]
+    assert (a >= one).to_list() == [True, True]
+    assert (a < one).to_list() == [False, False]
+    assert (a <= one).to_list() == [True, False]
+
+    one = date(2001, 1, 1)  # type: ignore
+    two = date(2001, 1, 2)  # type: ignore
+    a = pl.Series("a", [one, two])
+    assert (a == one).to_list() == [True, False]
+    assert (a != one).to_list() == [False, True]
+    assert (a > one).to_list() == [False, True]
+    assert (a >= one).to_list() == [True, True]
+    assert (a < one).to_list() == [False, False]
+    assert (a <= one).to_list() == [True, False]
+
+    # also test if the conversion stays correct with wide date ranges
+    one = date(201, 1, 1)  # type: ignore
+    two = date(201, 1, 2)  # type: ignore
+    a = pl.Series("a", [one, two])
+    assert (a == one).to_list() == [True, False]
+    assert (a == two).to_list() == [False, True]
+
+    one = date(5001, 1, 1)  # type: ignore
+    two = date(5001, 1, 2)  # type: ignore
+    a = pl.Series("a", [one, two])
+    assert (a == one).to_list() == [True, False]
+    assert (a == two).to_list() == [False, True]
+
+
+def test_truncate_negative_offset() -> None:
+    df = pl.DataFrame(
+        {
+            "event_date": [
+                datetime(2021, 4, 11),
+                datetime(2021, 4, 29),
+                datetime(2021, 5, 29),
+            ],
+            "adm1_code": [1, 2, 1],
+        }
+    )
+    out = df.groupby_dynamic(
+        time_column="event_date",
+        every="1mo",
+        period="2mo",
+        offset="-1mo",
+        include_boundaries=True,
+    ).agg(
+        [
+            pl.col("adm1_code"),
+        ]
+    )
+
+    assert out["event_date"].to_list() == [
+        datetime(2021, 4, 1),
+        datetime(2021, 4, 1),
+        datetime(2021, 5, 1),
+    ]
+    df = pl.DataFrame(
+        {
+            "event_date": [
+                datetime(2021, 4, 11),
+                datetime(2021, 4, 29),
+                datetime(2021, 5, 29),
+            ],
+            "adm1_code": [1, 2, 1],
+            "five_type": ["a", "b", "a"],
+            "actor": ["a", "a", "a"],
+            "admin": ["a", "a", "a"],
+            "fatalities": [10, 20, 30],
+        }
+    )
+
+    out = df.groupby_dynamic(
+        time_column="event_date",
+        every="1mo",
+        by=["admin", "five_type", "actor"],
+    ).agg([pl.col("adm1_code").unique(), (pl.col("fatalities") > 0).sum()])
+    assert out["event_date"].to_list() == [
+        datetime(2021, 4, 1),
+        datetime(2021, 5, 1),
+        datetime(2021, 4, 1),
+    ]
