@@ -1,9 +1,9 @@
 # flake8: noqa: W191,E101
 import sys
-import typing as tp
 from builtins import range
 from datetime import datetime
 from io import BytesIO
+from typing import Any, Iterator
 from unittest.mock import patch
 
 import numpy as np
@@ -649,11 +649,20 @@ def test_set() -> None:
     df["new"] = np.random.rand(10)
     df[df["new"] > 0.5, "new"] = 1
 
+    # set 2D
     df = pl.DataFrame({"b": [0, 0]})
-    df[["A", "B"]] = [[1, 2], [1, 2]]  # type: ignore
+    df[["A", "B"]] = [[1, 2], [1, 2]]
     assert df["A"] == [1, 1]
     assert df["B"] == [2, 2]
 
+    with pytest.raises(ValueError):
+        df[["C", "D"]] = 1
+    with pytest.raises(ValueError):
+        df[["C", "D"]] = [1, 1]
+    with pytest.raises(ValueError):
+        df[["C", "D"]] = [[1, 2, 3], [1, 2, 3]]
+
+    # set tuple
     df = pl.DataFrame({"b": [0, 0]})
     df[0, "b"] = 1
     assert df[0, "b"] == 1
@@ -661,6 +670,20 @@ def test_set() -> None:
     df[0, 0] = 2
     assert df[0, "b"] == 2
 
+    # row and col selection have to be int or str
+    with pytest.raises(ValueError):
+        df[:, [1]] = 1  # type: ignore
+    with pytest.raises(ValueError):
+        df[True, :] = 1  # type: ignore
+
+    # needs to be a 2 element tuple
+    with pytest.raises(ValueError):
+        df[(1, 2, 3)] = 1  # type: ignore
+
+    # we cannot index with any type, such as bool
+    with pytest.raises(NotImplementedError):
+        df[True] = 1  # type: ignore
+
 
 def test_melt() -> None:
     df = pl.DataFrame({"A": ["a", "b", "c"], "B": [1, 3, 5], "C": [2, 4, 6]})
@@ -1378,24 +1401,6 @@ def test_filter_with_all_expansion() -> None:
     assert out.shape == (2, 3)
 
 
-def test_diag_concat() -> None:
-    a = pl.DataFrame({"a": [1, 2]})
-    b = pl.DataFrame({"b": ["a", "b"], "c": [1, 2]})
-    c = pl.DataFrame({"a": [5, 7], "c": [1, 2], "d": [1, 2]})
-
-    out = pl.concat([a, b, c], how="diagonal")
-    expected = pl.DataFrame(
-        {
-            "a": [1, 2, None, None, 5, 7],
-            "b": [None, None, "a", "b", None, None],
-            "c": [None, None, 1, 2, 1, 2],
-            "d": [None, None, None, None, 1, 2],
-        }
-    )
-
-    assert out.frame_equal(expected, null_equal=True)
-
-
 def test_transpose() -> None:
     df = pl.DataFrame({"a": [1, 2, 3], "b": [1, 2, 3]})
     expected = pl.DataFrame(
@@ -1432,7 +1437,7 @@ def test_transpose() -> None:
     )
     assert expected.frame_equal(out)
 
-    def name_generator() -> tp.Iterator[str]:
+    def name_generator() -> Iterator[str]:
         base_name = "my_column_"
         count = 0
         while True:
@@ -1486,6 +1491,14 @@ def test_extension() -> None:
     assert sys.getrefcount(foos[0]) == base_count
 
 
+def test_groupby_order_dispatch() -> None:
+    df = pl.DataFrame({"x": list("bab"), "y": range(3)})
+    expected = pl.DataFrame({"x": ["b", "a"], "y_count": [2, 1]})
+    assert df.groupby("x", maintain_order=True).count().frame_equal(expected)
+    expected = pl.DataFrame({"x": ["b", "a"], "y_agg_list": [[0, 2], [1]]})
+    assert df.groupby("x", maintain_order=True).agg_list().frame_equal(expected)
+
+
 def test_schema() -> None:
     df = pl.DataFrame(
         {"foo": [1, 2, 3], "bar": [6.0, 7.0, 8.0], "ham": ["a", "b", "c"]}
@@ -1589,6 +1602,42 @@ def test_arithmetic() -> None:
     expected = pl.DataFrame({"a": [-1, 0], "b": [1, 2]})
     assert df_minus.frame_equal(expected)
 
+    df_mod = df % 2
+    expected = pl.DataFrame({"a": [1.0, 0.0], "b": [1.0, 0.0]})
+    assert df_mod.frame_equal(expected)
+
+    df2 = pl.DataFrame({"c": [10]})
+
+    out = df + df2
+    expected = pl.DataFrame({"a": [11.0, None], "b": [None, None]}).with_column(
+        pl.col("b").cast(pl.Float64)
+    )
+    assert out.frame_equal(expected, null_equal=True)
+
+    out = df - df2
+    expected = pl.DataFrame({"a": [-9.0, None], "b": [None, None]}).with_column(
+        pl.col("b").cast(pl.Float64)
+    )
+    assert out.frame_equal(expected, null_equal=True)
+
+    out = df / df2
+    expected = pl.DataFrame({"a": [0.1, None], "b": [None, None]}).with_column(
+        pl.col("b").cast(pl.Float64)
+    )
+    assert out.frame_equal(expected, null_equal=True)
+
+    out = df * df2
+    expected = pl.DataFrame({"a": [10.0, None], "b": [None, None]}).with_column(
+        pl.col("b").cast(pl.Float64)
+    )
+    assert out.frame_equal(expected, null_equal=True)
+
+    out = df % df2
+    expected = pl.DataFrame({"a": [1.0, None], "b": [None, None]}).with_column(
+        pl.col("b").cast(pl.Float64)
+    )
+    assert out.frame_equal(expected, null_equal=True)
+
 
 def test_getattr() -> None:
     df = pl.DataFrame({"a": [1.0, 2.0]})
@@ -1664,7 +1713,7 @@ def test_pivot_list() -> None:
 
 
 @pytest.mark.parametrize("as_series,inner_dtype", [(True, pl.Series), (False, list)])
-def test_to_dict(as_series: bool, inner_dtype: tp.Any) -> None:
+def test_to_dict(as_series: bool, inner_dtype: Any) -> None:
     df = pl.DataFrame(
         {
             "A": [1, 2, 3, 4, 5],
